@@ -180,7 +180,7 @@ class Graph(object):
     @property
     def n_all_edges(self):
         """n_items choose 2."""
-        return self.n_items * (self.n_items - 1) / 2
+        return self.n_items * (self.n_items - 1) // 2
 
     def neighbors(self, node: int) -> np.ndarray:
         """The indices of the neighbors of ``node``."""
@@ -226,8 +226,9 @@ class Graph(object):
                 "Graphs with negative edge weights cannot be drawn."
             )
 
-        if self.n_edges < 1e7 and self.n_all_edges > 1e7:
-            retain_fraction = 1e7 / self.n_all_edges
+        if self.n_edges < self.n_all_edges:
+            # copmute shortest paths, but no more than 1e7 of them ...
+            retain_fraction = min(1., 1e7 / self.n_all_edges)
             distance_graph = shortest_paths(
                 self, retain_fraction=retain_fraction, verbose=verbose
             )
@@ -550,6 +551,10 @@ def k_nearest_neighbors(
             "scipy.sparse adjacency matrix."
         )
 
+    if graph.n_edges == graph.n_all_edges:
+        # the graph is already full; no need to compute shortest paths.
+        graph_distances = False
+
     if graph_distances:
         graph = shortest_paths(graph, max_length=max_distance, verbose=verbose)
 
