@@ -12,6 +12,21 @@ from pymde.functions import penalties, losses
 from pymde import preprocess
 
 
+def _remove_anchor_anchor_edges(edges, data, anchors):
+    # exclude edges in which both items are anchors, since these
+    # items are already pinned in place by the anchor constraint
+    #
+    # TODO(akshayka): handle edge case in which every item is anchored,
+    # in which case the edge list will be empty after this exclusion
+    neither_anchors_mask = ~(
+        (edges[:, 0][..., None] == anchors).any(-1)
+        * (edges[:, 1][..., None] == anchors).any(-1)
+    )
+    edges = edges[neither_anchors_mask]
+    data = data[neither_anchors_mask]
+    return edges, data
+
+
 def preserve_distances(
     data,
     embedding_dim=2,
@@ -59,9 +74,9 @@ def preserve_distances(
 
     Arguments
     ---------
-    data: {np.ndarray, torch.Tensor, scipy.sparse matrix}(
-            shape=(n_items, n_features)), or pymde.Graph
-        A data matrix or a ``pymde.Graph`` instance.
+    data: {np.ndarray, torch.Tensor, scipy.sparse matrix} or pymde.Graph
+        The original data, a data matrix of shape ``(n_items, n_features)`` or
+        a graph.
     embedding_dim: int
         The embedding dimension.
     loss: pymde.Function class (or factory)
@@ -115,6 +130,10 @@ def preserve_distances(
         deviations = preprocess.scale(
             deviations, constraint.natural_length(n_items, embedding_dim)
         )
+    elif isinstance(constraint, constraints.Anchored):
+        edges, deviations = _remove_anchor_anchor_edges(
+            edges, deviations, constraint.anchors
+        )
 
     return problem.MDE(
         n_items=n_items,
@@ -165,11 +184,10 @@ def preserve_neighbors(
 
     Arguments
     ---------
-    data: {torch.Tensor, numpy.ndarray, scipy.sparse matrix}(
-            shape=(n_items, n_features)) or pymde.Graph
-        The original data, a data matrix or a graph. Neighbors are
-        computed using Euclidean distance if the data is a matrix,
-        or the shortest-path metric if the data is a graph.
+    data: {torch.Tensor, numpy.ndarray, scipy.sparse matrix} or pymde.Graph
+        The original data, a data matrix of shape ``(n_items, n_features)`` or
+        a graph. Neighbors are computed using Euclidean distance if the data is
+        a matrix, or the shortest-path metric if the data is a graph.
     embedding_dim: int
         The embedding dimension. Use 2 or 3 for visualization.
     attractive_penalty: pymde.Function class (or factory)
@@ -258,12 +276,22 @@ def preserve_neighbors(
     edges = knn_graph.edges.to(device)
     weights = knn_graph.weights.to(device)
 
+    if isinstance(constraint, constraints.Anchored):
+        # remove anchor-anchor edges before generating intialization
+        edges, weights = _remove_anchor_anchor_edges(
+            edges, weights, constraint.anchors
+        )
+
     if init == "quadratic":
         if verbose:
             problem.LOGGER.info("Computing quadratic initialization.")
         X_init = quadratic.spectral(
             n, embedding_dim, edges, weights, device=device
         )
+        if not isinstance(
+            constraint, (constraints._Centered, constraints._Standardized)
+        ):
+            constraint.project_onto_constraint(X_init, inplace=True)
     elif init == "random":
         X_init = constraint.initialization(n, embedding_dim, device)
     else:
@@ -275,28 +303,31 @@ def preserve_neighbors(
     if repulsive_penalty is not None:
         if repulsive_fraction is None:
             if isinstance(constraint, constraints._Standardized):
+                # standardization constraint already implicity spreads,
+                # so use a lower replusion
                 repulsive_fraction = 0.5
             else:
                 repulsive_fraction = 1
 
-        n_repulsive = int(repulsive_fraction * edges.shape[0])
         n_choose_2 = int(n * (n - 1) / 2)
-        if n_repulsive > n_choose_2:
-            problem.LOGGER.info(
-                f"The number of repulsive edges requested ({n_repulsive}) "
-                f"is larger than the total number of edges ({n_choose_2}). "
-                f"Sampling at most ({n_choose_2}) edges ..."
-            )
-            n_repulsive = n_choose_2
+        n_repulsive = int(repulsive_fraction * (edges.shape[0]))
+        # cannot sample more edges than there are available
+        n_repulsive = min(n_repulsive, n_choose_2 - edges.shape[0])
 
         negative_edges = preprocess.sample_edges(
             n, n_repulsive, exclude=edges
         ).to(device)
-        edges = torch.cat([edges, negative_edges])
 
         negative_weights = -torch.ones(
             negative_edges.shape[0], dtype=X_init.dtype, device=device
         )
+
+        if isinstance(constraint, constraints.Anchored):
+            negative_edges, negative_weights = _remove_anchor_anchor_edges(
+                negative_edges, negative_weights, constraint.anchors
+            )
+
+        edges = torch.cat([edges, negative_edges])
         weights = torch.cat([weights, negative_weights])
 
         f = penalties.PushAndPull(
@@ -330,3 +361,58 @@ def preserve_neighbors(
             dtype=mde._X_init.dtype,
         )
     return mde
+
+
+def laplacian_embedding(
+    data,
+    embedding_dim=2,
+    n_neighbors=None,
+    max_distance=None,
+    init="quadratic",
+    device="cpu",
+    verbose=False,
+) -> problem.MDE:
+    """Constructs an MDE problem for computing a Laplacian embedding.
+
+    The embedding preserves the nearest neighbors of each item, using
+    quadratic distortion functions and a standardization constraint.
+
+    Arguments
+    ---------
+    data: {torch.Tensor, numpy.ndarray, scipy.sparse matrix} or pymde.Graph
+        The original data, a data matrix of shape ``(n_items, n_features)`` or
+        a graph. Neighbors are computed using Euclidean distance if the data is
+        a matrix, or the shortest-path metric if the data is a graph.
+    embedding_dim: int
+        The embedding dimension. Use 2 or 3 for visualization.
+    n_neighbors: int (optional)
+        The number of nearest neighbors to compute for each row (item) of
+        ``data``. A sensible value is chosen by default, depending on the
+        number of items.
+    max_distance: float (optional)
+        If not None, neighborhoods are restricted to have a radius
+        no greater than ``max_distance``.
+    init: str
+        Initialization strategy; 'quadratic' or 'random'. If the quadratic
+        initialization takes too much time, try a random initialization.
+    device: str (optional)
+        Device for the embedding (eg, 'cpu', 'cuda').
+    verbose: bool
+        If ``True``, print verbose output.
+
+    Returns
+    -------
+    pymde.MDE
+        A ``pymde.MDE`` object, based on the original data.
+    """
+    return preserve_neighbors(
+        data,
+        embedding_dim=embedding_dim,
+        attractive_penalty=penalties.Quadratic,
+        repulsive_penalty=None,
+        n_neighbors=n_neighbors,
+        max_distance=max_distance,
+        init=init,
+        device=device,
+        verbose=verbose,
+    )
